package com.huguesjohnson;

/*
IniFile.java - Routines to read/write .ini files
Copyright  (C) 2000-2003 Hugues Johnson
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
the GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.lang.Boolean;
import java.lang.Byte;
import java.lang.Character;
import java.lang.Double;
import java.lang.Exception;
import java.lang.Float;
import java.lang.Integer;
import java.lang.Long;
import java.lang.NumberFormatException;
import java.lang.String;
import java.lang.StringBuffer;
import java.lang.System;
import java.util.Properties;

/** class IniFile reads/writes old-school style .ini files
 * recommended usage:
 *      private void readSavedSettings(){
 *          IniFile iniFile=new IniFile(this.settingsPath+File.separator+"YourAppName.ini");
 *          if(iniFile!=null){
 *              iniFile.read(); //must call .read() before trying to look up any values
 *              final String defaultString=new String("YourDefaultValue");
 *              String property1=new String(iniFile.getProperty("YourAppProperty",defaultString));
 *              String property2=new String(iniFile.getProperty("AnotherAppProperty",defaultString));
 *              //and so on..
 *          }
 *      }
 *      private void saveSettings(){
 *          IniFile iniFile=new IniFile(this.settingsPath+File.separator+"YourAppName.ini");
 *          if(iniFile!=null){
 *              iniFile.setProperty("YourAppProperty",this.someVariable);
 *              iniFile.setProperty("AnotherAppProperty",this.anotherVariable);
 *              iniFile.save("some header");
 *              iniFile=null;
 *          }
 *      }
 * @author Hugues Johnson
 */
public class IniFile{
    /** the name of the physical ini file */
    private String fileName;
    /** collection of properties stored in the file */
    private Properties properties;
    
    /** constructor
     * @param fileName name of ini file to read/write
     */
    public IniFile(String fileName){
        this.fileName=fileName;
        this.properties=new Properties();
    }
    
    /** reads the file specified in the constuctor
     * all data previously saved in memory is lost after read
     */
    public void read(){
        /* erase current contents of properties, if the file is not found the contents of properties should be empty*/
        this.properties=new Properties();
        try{
            FileInputStream fIn=new FileInputStream(this.fileName);
            this.properties.load(fIn);
            fIn.close();
        } catch(Exception x){
            x.printStackTrace();
        }
    }
    
    /** saves settings to file specfied in the constuctor - overwrites old file
     * @param header header for ini file
     */
    public void save(String header){
        try{
            /* does the directory this file should reside in exist? if it doesn't, a FileNoteFoundException will be thrown */
            File targetDirectory=new File(this.fileName.substring(0,this.fileName.lastIndexOf(File.separator)));
            if(!targetDirectory.exists()){
                targetDirectory.mkdir();
            }
            FileOutputStream fOut=new FileOutputStream(this.fileName);
            this.properties.store(fOut,header);
            fOut.close();
        } catch(Exception x){
            x.printStackTrace();
        }
    }
    
    /** clears out property list
     */
    public void clear(){
        if(this.properties!=null){
            this.properties.clear();
            this.properties=null;
        }
    }
    
    /** sets a property in memory - use save() to write all properties to file
     * @param name name (key) of the setting
     * @param value value of the setting
     */
    public void setProperty(String name,String value){
        this.properties.setProperty(name.trim(),value.trim());
    }
    
    /** sets a property in memory - use save() to write all properties to file
     * @param name name (key) of the setting
     * @param value value of the setting
     */
    public void setProperty(String name,int value){
        this.setProperty(name,String.valueOf(value));
    }
    
    /** sets a property in memory - use save() to write all properties to file
     * @param name name (key) of the setting
     * @param value value of the setting
     */
    public void setProperty(String name,boolean value){
        this.setProperty(name,String.valueOf(value));
    }
    
    /** sets a property in memory - use save() to write all properties to file
     * @param name name (key) of the setting
     * @param value value of the setting
     */
    public void setProperty(String name,long value){
        this.setProperty(name,String.valueOf(value));
    }
    
    /** sets a property in memory - use save() to write all properties to file
     * @param name name (key) of the setting
     * @param value value of the setting
     */
    public void setProperty(String name,float value){
        this.setProperty(name,String.valueOf(value));
    }
    
    /** sets a property in memory - use save() to write all properties to file
     * @param name name (key) of the setting
     * @param value value of the setting
     */
    public void setProperty(String name,double value){
        this.setProperty(name,String.valueOf(value));
    }
    
    /** sets a property in memory - use save() to write all properties to file
     * @param name name (key) of the setting
     * @param value value of the setting
     */
    public void setProperty(String name,char value){
        this.setProperty(name,String.valueOf(value));
    }
    
    /** sets a property in memory - use save() to write all properties to file
     * @param name name (key) of the setting
     * @param value value of the setting
     */
    public void setProperty(String name,byte value){
        this.setProperty(name,String.valueOf(value));
    }
    
    /** retrieves a property from memory, use read() to load all properties from file into memory
     * @param name name (key) of the property to retrieve
     * @param defaultValue value to return if property is not found
     * @return value of the property
     */
    public String getProperty(String name,String defaultValue){
        return(this.properties.getProperty(name,defaultValue).trim());
    }
    
    /** retrieves a property from memory, use read() to load all properties from file into memory
     * @param name name (key) of the property to retrieve
     * @param defaultValue value to return if property is not found
     * @return value of the property
     */
    public int getProperty(String name,int defaultValue){
        String property=this.properties.getProperty(name,String.valueOf(defaultValue)).trim();
        try{
            int returnValue=Integer.valueOf(property).intValue();
            return(returnValue);
        } catch(NumberFormatException nfx){
            return(defaultValue);
        }
    }
    
    /** retrieves a property from memory, use read() to load all properties from file into memory
     * @param name name (key) of the property to retrieve
     * @param defaultValue value to return if property is not found
     * @return value of the property
     */
    public boolean getProperty(String name,boolean defaultValue){
        String property=this.properties.getProperty(name,String.valueOf(defaultValue)).trim();
        try{
            boolean returnValue=Boolean.valueOf(property).booleanValue();
            return(returnValue);
        } catch(NumberFormatException nfx){
            return(defaultValue);
        }
    }
    
    /** retrieves a property from memory, use read() to load all properties from file into memory
     * @param name name (key) of the property to retrieve
     * @param defaultValue value to return if property is not found
     * @return value of the property
     */
    public double getProperty(String name,double defaultValue){
        String property=this.properties.getProperty(name,String.valueOf(defaultValue)).trim();
        try{
            double returnValue=Integer.valueOf(property).doubleValue();
            return(returnValue);
        } catch(NumberFormatException nfx){
            return(defaultValue);
        }
    }
    
    /** retrieves a property from memory, use read() to load all properties from file into memory
     * @param name name (key) of the property to retrieve
     * @param defaultValue value to return if property is not found
     * @return value of the property
     */
    public byte getProperty(String name,byte defaultValue){
        String property=this.properties.getProperty(name,String.valueOf(defaultValue)).trim();
        try{
            byte returnValue=Byte.valueOf(property).byteValue();
            return(returnValue);
        } catch(NumberFormatException nfx){
            return(defaultValue);
        }
    }
    
    /** retrieves a property from memory, use read() to load all properties from file into memory
     * @param name name (key) of the property to retrieve
     * @param defaultValue value to return if property is not found
     * @return value of the property
     */
    public long getProperty(String name,long defaultValue){
        String property=this.properties.getProperty(name,String.valueOf(defaultValue)).trim();
        try{
            long returnValue=Long.valueOf(property).longValue();
            return(returnValue);
        } catch(NumberFormatException nfx){
            return(defaultValue);
        }
    }
    
    /** retrieves a property from memory, use read() to load all properties from file into memory
     * @param name name (key) of the property to retrieve
     * @param defaultValue value to return if property is not found
     * @return value of the property
     */
    public float getProperty(String name,float defaultValue){
        String property=this.properties.getProperty(name,String.valueOf(defaultValue)).trim();
        try{
            float returnValue=Float.valueOf(property).floatValue();
            return(returnValue);
        } catch(NumberFormatException nfx){
            return(defaultValue);
        }
    }
    
    /** retrieves a property from memory, use read() to load all properties from file into memory
     * @param name name (key) of the property to retrieve
     * @param defaultValue value to return if property is not found
     * @return value of the property
     */
    public char getProperty(String name,char defaultValue){
        String property=this.properties.getProperty(name,String.valueOf(defaultValue)).trim();
        if(property.length()>1){
            return(defaultValue);
        } else{
            return(property.charAt(0));
        }
    }
    
    /** returns a String representation of this object
     * @return String representation of this object
     */
    public String toString(){
        StringBuffer tostring=new StringBuffer(super.toString());
        tostring.append("\nfileName=");
        tostring.append(this.fileName);
        return(tostring.toString());
    }
}